package uremote;

import java.io.Serializable;
import java.security.InvalidParameterException;

/**
 * A class that descibes a change in the universal remote GUI's state.
 *
 * @author Greg Hackmann
 */
public class ActionInfo implements Serializable
{
	/** (See {@link #getType()}) */
	public static final int VALUE_TYPE = 1;
	/** (See {@link #getType()}) */
	public static final int TOGGLE_TYPE = 2;
	
	private int type;
	private String functionName;
	private int newVal;
	private boolean newToggle;
	
	/**
	 * Instantiate an advertisement of a clicked slider.
	 *
	 * @param functionName the name of the button clicked
	 * @param newVal the value of the slider
	 */
	public ActionInfo(String functionName, int newVal)
	{
		this.functionName = functionName;
		this.newVal = newVal;
		type = VALUE_TYPE;
	}
	
	/**
	 * Instantiate an advertisement of a clicked button.
	 *
	 * @param functionName the name of the button clicked
	 * @param newToggle the value of the button (false if unlit, true if lit
	 * or if button is not a toggle)
	 */
	public ActionInfo(String functionName, boolean newToggle)
	{
		this.functionName = functionName;
		this.newToggle = newToggle;
		type = TOGGLE_TYPE;
	}
	
	/**
	 * Get the type of action generated.
	 *
	 * @return {@link #VALUE_TYPE} if the action was performed on a slider,
	 * or {@link #TOGGLE_TYPE} if the action was performed on a button
	 */
	public int getType() { return type; }
	
	/**
	 * Get the function name of the component altered.
	 *
	 * @return the component's function name
	 */
	public String getFunctionName() { return functionName; }
	
	/**
	 * Get the new integer value of the component.
	 *
	 * @return the new value
	 * @throws InvalidParameterException if this action was not performed on a slider
	 */
	public int getIntValue()
	{
		if(type != VALUE_TYPE) throw new InvalidParameterException("Tried to read int value from boolean action");
		return newVal;
	}

	/**
	 * Get the new boolean value of the component.
	 *
	 * @return the new value
	 * @throws InvalidParameterException if this action was not performed on a button
	 */
	public boolean getBooleanValue()
	{
		if(type != TOGGLE_TYPE) throw new InvalidParameterException("Tried to read boolean value from int action");
		return newToggle;
	}
}
