package uremote;

/**
 * This interface describes a device definition that can be plugged into 
 * {@link GenericDeviceAgent}.
 * <p>
 * Example implementation:
 * <p>
 * <tt>
 * public class TrivialDevice implements DeviceDefinition<br>
 * {<ul>
 * 		private int value = 0;<br>
 * 		<br>
 * 		public String getName()<br>
 * 		{<ul>
 * 			return "Trivial";</ul>
 * 		}<br>
 * 	 	<br>
 * 		public String getID()<br>
 * 	 	{<ul>
 * 			return "Trivial1328954";</ul>
 * 	 	}<br>
 * 	 	<br>
 * 	 	public String getIcon()<br>
 *  	{<ul>
 * 			return "COMPUTER";</ul>
 *  	}<br>
 *  	<br>
 *  	public void setAgent(GenericDeviceAgent agent) { }<br>
 *  	<br>
 * 		public void reactToButton(String functionName, boolean toggle)<br>
 * 		{<ul>
 * 			if(functionName.equals("echo"))<ul>
 * 				System.out.println(value);</ul></ul>
 * 		}<br>
 *  	<br>
 * 		public void reactToSlider(String functionName, int value)<br>
 * 		{<ul>
 * 			if(functionName.equals("slider"))<ul>
 * 				this.value = value;</ul></ul>
 * 		}<br>
 *  	<br>
 * 		public String [] getFunctions()<br>
 * 		{<ul>
 * 			return new String [] { "echo", "slider" };</ul>
 * 		}<br>
 *  	<br>
 * 		public String getHelpText(String functionName)<br>
 * 		{<ul>
 * 			if(functionName.equals("echo"))<ul>
 * 				return "Print the slider's value";</ul>
 * 			else if(functionName.equals("slider"))<ul>
 * 				return "The value to print";</ul>
 * 			else return null;</ul>
 * 		}<br>
 *  	<br>
 * 		public ComponentAdvertisement getAdvertisement(String functionName)<br>
 * 		{<ul>
 * 			if(functionName.equals("echo"))<ul>
 * 				return new ButtonAdvertisement("echo", false, "PLAY", 0, 0);</ul>
 * 			else if(functionName.equals("slider))<ul>
 * 				return new SliderAdvertisement("slider", 0, 100, 0, 1, 5, 1);</ul>
 * 			else return null;</ul>
 * 		}<br>
 *  	<br>
 *  	public ComponentState getState(String functionName)<br>
 *  	{<ul>
 * 			if(functionName.equals("echo"))<ul>
 * 				return new ButtonState(true, false);</ul>
 * 			else if(functionName.equals("slider))<ul>
 * 				return new SliderState(true, "Print Me", value);</ul>
 * 			else return null;</ul>
 *  	}<br>
 *  	<br>
 * 		public void shutDown() { }</ul>
 * }</tt>
 * 
 * @author Greg Hackmann
 */
public interface DeviceDefinition
{
	/**
	 * This method should return the device's ID, which is fixed.  This will
	 * be displayed in the remote's GUI to identify the device.
	 *
 	 * @return the device's name
	 */
	public String getName();

	/**
	 * This method should return the device's ID, which is fixed.  This should
	 * be unique for each device.
	 *
	 * @return the device's icon
	 */
	public String getID();
	
	/**
	 * This method should return the device's icon
	 * (see {@link uremote.swt.ui.RemoteIconManager#deviceFilename(String)}),
	 * which is fixed.
	 *
	 * @return the device's ID
	 */
	public String getIcon();

	/**
	 * This method will be called by the {@link GenericDeviceAgent} to give a
	 * reference to itself to the device.  Since most devices will need to
	 * call methods on GenericDeviceAgent, the device should keep this reference
	 * in an instance variable.
	 *
	 * @param agent the GenericDeviceAgent using this device definition
	 */
	public void setAgent(GenericDeviceAgent agent);
	
	/**
	 * This method should react to a button being pressed.
	 *
	 * @param functionName the function ID of the button pressed
	 * @param toggle the value of the button (false if unlit, true if lit or if
	 * the button is not a toggle button)
	 */
	public void reactToButton(String functionName, boolean toggle);

	/**
	 * This method should react to a slider being moved.
	 *
	 * @param functionName the function ID of the slider moved
	 * @param value the slider's new value
	 */
	public void reactToSlider(String functionName, int value);
	
	/**
	 * This method should return a list of the functions that this device
	 * supports.  All the functions in this array must have a
	 * corresponding return value for {@link #getHelpText(String)},
	 * {@link #getAdvertisement(String)}, and {@link #getState(String)}.
	 *
	 * @return an array of function IDs
	 */
	public String [] getFunctions();
	
	/**
	 * This method should return the "help" text for the given component.
	 * All components must have some "help" text; those that do not should
	 * return an empty String.
	 *
	 * @param functionName the function ID of the component whose "help" text
	 * is being requested
	 * @return the "help" text for that component
	 */
	public String getHelpText(String functionName);
	
	
	/**
	 * This method should return the advertisement for the given component.
	 * All components must have some advertisement.
	 *
	 * @param functionName the function ID of the component whose advertisement
	 * is being requested
	 * @return the advertisement for that component
	 */
	public ComponentAdvertisement getAdvertisement(String functionName);
	
	/**
	 * This method should return the current state for the given component.
	 * All components must have some state.
	 *
	 * @param functionName the function ID of the component whose state
	 * is being requested
	 * @return the state for that component
	 */
	public ComponentState getState(String functionName);
	
	/**
	 * This method is called by the {@link GenericDeviceAgent} when the agent
	 * is shutting down.  If the device needs to do any "cleaning up", it should
	 * be done in this method.
	 */
	public void shutDown();
}
