package uremote;

import java.util.*;
import limone.*;
import eLights.*;
import eBeacon.*;

/**
 * An {@link Agent} used to advertise a device's capabilities to a universal
 * remote control and respond to the remote's GUI.
 *
 * @author Greg Hackmann
 */
public class GenericDeviceAgent extends Agent implements ReactionListener
{
	private DeviceDefinition device;
	
	/**
	 * Instantiate the Agent with the given ID and class name.  The class name
	 * should be a fully-qualified name of a {@link DeviceDefinition} that
	 * describes the device to be advertised and controlled.
	 *
	 * @param id the Agent's ID
	 * @param className the Agent's class name
	 */
	public GenericDeviceAgent(AgentID id, String className)
	{
		super(id, className);
		try
		{
			device = instantiate(className);
			device.setAgent(this);
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.exit(0);
		}
		/* Try to instantiate the device's definition */
		
		disengagement_threshold = remote_operation_timeout = 10000;
		/* Make the timeout threshold really big for slower devices' sake */
	}
	
	private DeviceDefinition instantiate(String className) throws Exception
	{
		Class myClass = Class.forName(className);
		DeviceDefinition returnMe = (DeviceDefinition)myClass.newInstance();
		return returnMe;
	}
	
	protected void shutDown()
	{
		device.shutDown();
		/* Tell the device the agent's being shut down */
	}

	public void agentRun()
	{
	}

	public void agentInit()
	{
		ETemplate actTemplate = new ETemplate();
		actTemplate.addConstraint(new EConstraint("ActionTuple",
							     ActionInfo.class, new DefaultConstraintFunction()));

		ProfileSelector ps = new ProfileSelector();
		ps.addConstraint(new ProfileConstraint("_AgentID", AgentID.class,
						 new EquivalencyConstraintFunction(getAgentID())));

		ReactivePattern actPattern = new ReactivePattern(ps, Reaction.ONCE_PER_TUPLE, actTemplate);
		rr.registerReaction(new Reaction(actPattern, this));
		/* Listen in on ActionTuples */
		
		String [] functions = device.getFunctions();
		/* Get the functions to advertise */
		
		Hashtable adTable = new Hashtable();
		for(int i = 0; i < functions.length; i++)
			adTable.put(functions[i], device.getAdvertisement(functions[i]));
		AdvertisementInfo ad = new AdvertisementInfo(device.getName(), device.getID(), device.getIcon(), adTable);
		AdvertisementTuple adTuple = new AdvertisementTuple(ad);
		/* Get all the ads and put them in an AdvertisementTuple */
	
		Hashtable helpTable = new Hashtable();
		for(int i = 0; i < functions.length; i++)
			helpTable.put(functions[i], device.getHelpText(functions[i]));
		HelpInfo help = new HelpInfo(device.getID(), helpTable);
		HelpTuple helpTuple = new HelpTuple(help);
		/* Do the same for the help text */
		
		Hashtable stateTable = new Hashtable();
		for(int i = 0; i < functions.length; i++)
			stateTable.put(functions[i], device.getState(functions[i]));
		StateInfo state = new StateInfo(device.getID(), stateTable);
		StateTuple stateTuple = new StateTuple(state);
		/* And the state info */

		localSendTuple(adTuple);
		localSendTuple(helpTuple);
		localSendTuple(stateTuple);
		/* Out all three tuples into our local tuple space */
	}

	public void agentDone()
	{
	}
	
	public void reactsTo(ReactionEvent re)
	{
		ActionInfo info = (ActionInfo)re.tuple().getField("ActionTuple").getValue();
		if(info == null) return;
		/* We only care about ActionTuples */
		
		if(info.getType() == ActionInfo.TOGGLE_TYPE)
			device.reactToButton(info.getFunctionName(), info.getBooleanValue());
		else if(info.getType() == ActionInfo.VALUE_TYPE)
			device.reactToSlider(info.getFunctionName(), info.getIntValue());
		/* Tell the device about the action */
	}

	/**
	 * Put the given {@link ETuple} into the agent's local tuple space.
	 *
	 * @param t the tuple to put into the local tuple space
	 */
	public void localSendTuple(ETuple t)
	{
		tupleSpace.out(t);
	}
	
	/**
	 * Put a tuple into the device's local tuple space that represents a change
	 * in one of the component's states.
	 *
	 * @param function the function name of the component whose state has
	 * changed
	 * @param newState the new state of the component
	 */
	public void changeState(String function, ComponentState newState)
	{
		changeState(new String [] { function }, new ComponentState [] { newState });
	}
	
	
	/**
	 * Put a tuple into the device's local tuple space that represents a change
	 * in several of the components' states.  The function names and states
	 * should be passed in as equally-sized arrays, where functions[i]
	 * corresponds to state[i] for all i.
	 *
	 * @param functions the function names of the components whose states have
	 * changed
	 * @param newStates the new states of the components
	 */
	public void changeState(String [] functions, ComponentState [] newStates)
	{
		ETemplate stateTemplate = new ETemplate();
		stateTemplate.addConstraint(new EConstraint("StateTuple",
							     StateInfo.class, new DefaultConstraintFunction()));
		
		try
		{
			ETuple stateTuple = tupleSpace.in(stateTemplate);
			
			StateInfo stateInfo = (StateInfo)stateTuple.getField("StateTuple").getValue();
			for(int i = 0; i < functions.length; i++)
				stateInfo.getFunctionToState().put(functions[i], newStates[i]);
			
			tupleSpace.out(stateTuple);
			/* Put the state changes into a tuple and out it */
		}
		catch(ETupleSpaceException e)
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * Close the Agent.
	 */
	public void closeAgent()
	{
		LimoneServer.getServer().killAgent(getAgentID());
	}
}