package uremote;

import java.util.Hashtable;
import eBeacon.*;
import eLights.*;
import limone.*;

/**
 * An {@link Agent} used to display a universal remote control.  In order to
 * use this agent, it must be extended, and the {@link #instantiateGUIManager()}
 * method must be implemented.
 *
 * @author Greg Hackmann
 */
public abstract class URemoteAgent extends Agent implements ReactionListener, AcquaintanceChangedListener
{
	protected URemoteGUIManager gm;
	private Hashtable idTable = new Hashtable();
	
	/**
	 * Instantiate the Agent with the given ID and name.
	 *
	 * @param id the Agent's ID
	 * @param name the Agent's name
	 */
	public URemoteAgent(AgentID id, String name)
	{
		super(id, name);
		aql.addChangeListener(this);
		/* We want to know about new devices in our range */
		
		instantiateGUIManager();
		/* Make the GUI */
		
		disengagement_threshold = remote_operation_timeout = 10000;
		/* Make the timeout pretty big for the sake of slow devices */
	}
	
	/**
	 * Instantiate the {@link uremote.URemoteGUIManager} specific to the GUI
	 * type.  All classes that extend this Agent must implement this method.
	 */
	protected abstract void instantiateGUIManager();

	protected void shutDown()
	{
	}

	public void agentRun()
	{
	}
	
	public void agentInit()
	{
	}
	
	public void agentDone()
	{
	}

	public void reactsTo(ReactionEvent re)
	{
		gm.update((StateInfo)re.tuple().getField("StateTuple").getValue());
		/* We only react to StateTuples, so send it over to the GUI */
	}


	public void acuaintanceChanged(AcquaintanceChangedEvent ace)
	{
		Profile profile = ace.getProfile();
		AgentID id = (AgentID)(profile.getProperty("_AgentID").getValue());
		
		/* If the agent is being removed, tell the GUI so,
		   and remove its ID from our list */
		if(ace.getType() == AcquaintanceChangedEvent.AGENT_REMOVED)
		{
			//System.out.println("Why is " + id.getName() + " disengaging?");
			String deviceID = (String)idTable.get(id);
			if(deviceID != null) gm.deleteDevice(deviceID);
			return;
		}
		
		/* We want to react to this agent OUTing new StateTuples */
		ETemplate stateTemplate = new ETemplate();
		stateTemplate.addConstraint(new EConstraint("StateTuple",
							        StateInfo.class, new DefaultConstraintFunction()));

		ProfileSelector ps = new ProfileSelector();
		ps.addConstraint(new ProfileConstraint("_AgentID", AgentID.class,
						 new EquivalencyConstraintFunction(id)));
						 
		ReactivePattern statePattern = new ReactivePattern(ps, Reaction.ONCE_PER_TUPLE, stateTemplate);
		
		/* Now get a proxy so we can read in its advertised services */
		AgentProxy proxy = aql.getAgentProxy(id);
		if(proxy == null) return;
		
		ETuple adTuple = null, helpTuple = null, stateTuple = null;
		
		/* The agent should have an AdvertisementTuple, a HelpTuple, and a
		   StateTuple, so read them all in */
		try
		{
			//System.out.println("Trying to engage with " + id.getName());
			ETemplate adTemplate = new ETemplate();
			adTemplate.addConstraint(new EConstraint("AdvertisementTuple",
								     AdvertisementInfo.class, new DefaultConstraintFunction()));
			adTuple = proxy.rdp(adTemplate);
			if(adTuple == null)
			{
			//	System.out.println("No adTuple");
				return;
			}
			
			ETemplate helpTemplate = new ETemplate();
			helpTemplate.addConstraint(new EConstraint("HelpTuple",
									   HelpInfo.class, new DefaultConstraintFunction()));
			helpTuple = proxy.rdp(helpTemplate);
			if(helpTuple == null)
			{
			//	System.out.println("No helpTuple");
				return;
			}
			
			stateTuple = proxy.rdp(stateTemplate);
			if(stateTuple == null)
			{
			//	System.out.println("No stateTuple");
				return;
			}
		}
		catch(RejectionException e) { e.printStackTrace(); return; }
		catch(TimeOutException e) { System.out.println(e); return; }
				
		AdvertisementInfo adInfo = (AdvertisementInfo)adTuple.getField("AdvertisementTuple").getValue();
		HelpInfo helpInfo = (HelpInfo)helpTuple.getField("HelpTuple").getValue();
		/* Get the info classes from the tuples */
			
//		if(idTable.containsKey(adInfo.getDeviceID())) return;
		/* If we've already got one, return */
		
		gm.addDevice(id, adInfo, helpInfo);
		/* Otherwise tell the GUI we've got a new device */
				
		StateInfo stateInfo = (StateInfo)stateTuple.getField("StateTuple").getValue();
		gm.update(stateInfo);
		/* Now update the GUI to the device's current state */
				
		idTable.put(id, adInfo.getDeviceID());
		/* Remember the agent ID for later */
		
		rr.registerReaction(new Reaction(statePattern, URemoteAgent.this));
	}
}
