package uremote.awt.ui;
import java.awt.*;
import java.util.*;

/**
 * A common class that all components to be shown on the display should
 * extend.
 *
 * @author Greg Hackmann
 */
public abstract class RemoteComponent extends Panel
{
	private String id;
	private GridBagConstraints constraints;
	private String text = "", helpText;
	private Vector clickListeners;
	private boolean clickable = true, viewable = true, lit = false;
	private Container parent;
	
	/**
	 * Instantiate the component to the given size.
	 *
	 * @param parent the {@link java.awt.Container} that will contain this component
	 * @param id the function name of the component
	 * @param x the X coordinate of the component, in grid squares
	 * @param y the Y coordinate of the component, in grid squares
	 * @param w the width of the component, in grid squares
	 * @param h the height of the component, in grid squares
	 */
	public RemoteComponent(Container parent, String id, int x, int y, int w, int h)
	{
		super(new GridLayout());
		this.id = id;
		this.parent = parent;

		constraints = new GridBagConstraints();
		constraints.fill = GridBagConstraints.BOTH;
		constraints.gridx = x;
		constraints.gridy = y;
		constraints.gridwidth = w;
		constraints.gridheight = h;
		constraints.weightx = constraints.weighty = 1.0;
		
//		setBackground(Color.LIGHT_GRAY);
//		setForeground(Color.BLACK);

		((GridBagLayout)parent.getLayout()).setConstraints(this, constraints);
		parent.add(this);
//		parent.add(this, constraints);
	}
	
	/**
	 * Set the component's text.  Not all component may use this label.
	 *
	 * @param t the text to use
	 */
	public void setText(String t) { text = t; }
	
	/**
	 * Get the component's text.
	 *
	 * @return the component's text
	 */
	public String getText() {  return text; }
	
	/**
	 * Set the component's "help" text, which is shown on the help page.
	 *
	 * @param t the text to use
	 */
	public void setHelpText(String t) { helpText = t; }
	
	/**
	 * Get the component's "help" text.
	 *
	 * @return the component's text
	 */
	public String getHelpText() { return helpText; }
	
	/**
	 * Set the width of the component.
	 *
	 * @param w the width in grid squares
	 */
	public void setGridWidth(int w) { constraints.gridwidth = w; resize(); }
	
	/**
	 * Get the width of the component.
	 *
	 * @return the width in grid squares
	 */
	public int getGridWidth() { return constraints.gridwidth; }
	
	/**
	 * Set the height of the component.
	 *
	 * @param h the height in grid squares
	 */
	public void setGridHeight(int h) { constraints.gridheight = h; resize(); }
	
	/**
	 * Get the height of the component.
	 *
	 * @return the height in grid squares
	 */
	public int getGridHeight() { return constraints.gridheight; }

	/**
	 * Set whether or not this component is enabled.  {@link #setEnabled(boolean)} is
	 * reserved for use by AWT and should not be used for this.
	 *
	 * @param flag true if enabled, false if disabled
	 */
	public void setClickable(boolean flag)
	{
		clickable = flag;
		setEnabled(clickable);
	}
	
	/**
	 * Get whether or not this component is enabled.
	 *
	 * @return true if enabled, false if disabled
	 */
	public boolean isClickable() { return clickable; }
	
	/**
	 * Set whether or not this component is visible.  {@link #setVisible(boolean)}
	 * is reserved for use by AWT and should not be used for this.
	 *
	 * @param flag true if visible, false if invisible
	 */
	public void setViewable(boolean flag)
	{
		viewable = flag;
		setVisible(flag);
	}
	/**
	 * Get whether or not this component is visible.
	 *
	 * @return true if visible, false if invisible
	 */
	public boolean isViewable() { return viewable; }

	/**
	 * Set whether or not this component is lit.
	 *
	 * @param flag true if lit, false if unlit
	 */
	public void setLit(boolean flag) { lit = flag; }
	/**
	 * Get whether or not this component is lit.
	 *
	 * @return true if lit, false if unlit
	 */
	public boolean isLit() { return lit; }
	
	/**
	 * Return the function name (ID) of this component.
	 *
	 * @return the function name of the current component
	 */
	public String getFunctionName() { return id; }
	
	private void resize()
	{
		parent.remove(this);
		parent.add(this, constraints);
	}
}

