package uremote.swt;

import java.io.*;
import java.util.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.*;

import uremote.swt.ui.*;
import uremote.*;
import eLights.*;
import limone.*;

/**
 * A helper class used to manage the universal remote control's GUI.
 *
 * @author Greg Hackmann
 */
public class SWTGUIManager implements URemoteGUIManager
{
	private Hashtable pageTable;
	private RemoteDisplay display;
	private URemoteAgent agent;
	
	/**
	 * Instantiate a GUI listening to the given Agent.
	 *
	 * @param agent the URemoteAgent that will manipulate this GUI
	 */
	public SWTGUIManager(URemoteAgent agent)
	{
		this.agent = agent;
		pageTable = new Hashtable();
		startDisplay();
	}
	
	public SWTGUIManager(URemoteAgent agent, RemoteDisplay display)
	{
		this.agent = agent;
		pageTable = new Hashtable();
		this.display = display;
	}
	
	private void startDisplay()
	{
		Thread displayThread = new Thread()
		{
			public void run()
			{
				display = new RemoteDisplay();
				display.onClose(new Runnable()
				{
					public void run()
					{
						savePages();
						LimoneServer.getServer().killAgent(agent.getAgentID());
						LimoneServer.getServer().shutdown();
						/* When the remote is closed, save the open pages
						   and stop the Limone server */
					}
				});
				
				String [] preload = { "FASTFORWARD", "REWIND", "PLAY", "STOP",
						              "PAUSE", "PREVIOUS", "NEXT" };
				for(int i = 0; i < preload.length; i++)
				{
					String iconBase = RemoteIconManager.iconFilename(preload[i]);
					display.loadImage(iconBase + ".png");
					display.loadImage(iconBase + "-lit.png");
					display.loadImage(iconBase + "-disabled.png");
					display.loadImage(iconBase + "-icon.png");
				}
				/* Preload frequently-used images */
				
				display.open();
			}
		};
		
		displayThread.start();
		/* This has to be in its own thread since the main loop makes
		   RemoteDisplay.open(); block until the shell is closed */
	}
	
	private boolean isRunning()
	{
		return display == null || display.isOpen();
	}
	
	private void waitForDisplay()
	{
		while(display == null)
		{
			try
			{
				Thread.sleep(100);
			}
			catch (InterruptedException e)
			{
				e.printStackTrace();
			}
		}
		/* TODO: is there a better way to do this? */
	}
	
	private void savePages()
	{
		Enumeration keys = pageTable.keys();
		while(keys.hasMoreElements())
		{
			String currentID = (String)keys.nextElement();
			
			RemotePage current = (RemotePage)pageTable.get(currentID);
			if(current == null) continue;
			
			savePage(currentID, current);
		}
		/* Gp though the list of pages and save all of the valid ones */
	}
	
	private void savePage(String id, RemotePage page)
	{
		String filename = URemoteConstants.path + "layout/" + id + ".dat";
		
		try
		{
			ObjectOutputStream out = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(filename)));
			out.writeObject(page.getPanel().getLayoutInfo());
			out.flush();
			out.close();
			/* Write the page's layout info to a file */
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * Add the specified device to the GUI.
	 *
	 * @param id the device's Agent's ID
	 * @param ad the device's advertisement information
	 * @param help the device's help information
	 */
	public void addDevice(final AgentID id, final AdvertisementInfo ad, final HelpInfo help)
	{
		waitForDisplay();
		display.runCommand(new Runnable()
		{
			public void run()
			{
				runAddDevice(id, ad, help);
			}
		});
	}
	
	private void runAddDevice(final AgentID id, AdvertisementInfo ad, HelpInfo help)
	{
		Hashtable buttonTable = ad.getAllFunctionInfo();
		Enumeration enum = buttonTable.elements();
		Vector components = new Vector();
		String deviceName = ad.getDeviceName();
		String deviceID = ad.getDeviceID();
		Hashtable layoutTable = null;
		
		/* Look for a file containing layout information */
		File inFile = new File(URemoteConstants.path + "layout/" + deviceID + ".dat");
		if(inFile.exists())
		{
			try
			{
				ObjectInputStream in = new ObjectInputStream(new BufferedInputStream(new FileInputStream(inFile)));
				layoutTable = (Hashtable)in.readObject();
				in.close();
			}
			catch(IOException e)
			{
				e.printStackTrace();
				layoutTable = null;
			}
			catch(ClassNotFoundException e)
			{
				e.printStackTrace();
				layoutTable = null;
			}
		}
	
		int maxY = -1;

		/* Find the largest Y value of all the buttons and put the
		   buttons in a Vector */
		while(enum.hasMoreElements())
		{
			ComponentAdvertisement current = (ComponentAdvertisement)enum.nextElement();
			components.addElement(current);
			int y = current.getY();
			int height = current.getHeight();
			if(layoutTable != null)
			{
				PersistentLayout layout = (PersistentLayout)layoutTable.get(current.getFunctionName());
				if(layout != null)
				{
					y = layout.y;
					height = layout.height;
				}
			}
			if(y + height - 1 > maxY) maxY = y + height - 1;
		}
		maxY++;
		/* Now assemble the grid from the buttons' (x, y) */
		ComponentAdvertisement [][] grid = new ComponentAdvertisement[5][maxY + 1];
		for(int y = 0; y < maxY + 1; y++)
		{
			for(int x = 0; x < 5; x++)
			{
				grid[x][y] = null;
			}
		}
		for(enum = components.elements(); enum.hasMoreElements();)
		{
			ComponentAdvertisement current = (ComponentAdvertisement)enum.nextElement();
			int x = current.getX(), y = current.getY();
			if(layoutTable != null)
			{
				PersistentLayout layout = (PersistentLayout)layoutTable.get(current.getFunctionName());
				if(layout != null)
				{
					x = layout.x; y = layout.y;
				}
			}
			grid[x][y] = current;
		}

		/* Make a page to put the buttons in */
		RemotePage page = display.makePage();
//		page.hide();
		String deviceIcon = RemoteIconManager.deviceFilename(ad.getDeviceIcon());
		if(deviceIcon != null) page.setImage(display.loadImage(deviceIcon));
		page.setText(deviceName);
		
		/* Fill in the page with the buttons in the grid */
		for(int y = 0; y < maxY + 1; y++)
		{
			for(int x = 0; x < 5; x++)
			{
				final ComponentAdvertisement current = grid[x][y];
				int width = 1, height = 1;
				boolean viewable = true;

				/* If the space isn't blank, try to override the size and
				   visibility from information in the layout hash table */
				if(current != null)
				{
					width = current.getWidth();
					height = current.getHeight();
					
					if(layoutTable != null)
					{
						PersistentLayout layout = (PersistentLayout)layoutTable.get(current.getFunctionName());
						if(layout != null)
						{
							width = layout.width;
							height = layout.height;
							viewable = layout.viewable;
						}
					}
				}
				else page.makeBlank();
				
				if(current instanceof ButtonAdvertisement)
				{
					ButtonAdvertisement buttonAd = (ButtonAdvertisement)current;
					final RemoteButton button = page.makeButton(current.getFunctionName(), width, height);
					button.setText(current.getFunctionName());
					button.setViewable(viewable);
					/* Make the button and fill in its text and visibility */
					
					String helpText = help.getFunctionHelp(current.getFunctionName());
					if(helpText != null) button.setHelpText(helpText);
					/* Fill in its help text */
					
					String iconBase = RemoteIconManager.iconFilename(buttonAd.getIcon());
					if(iconBase != null)
					{
						button.setImage(display.loadImage(iconBase + ".png"));
						button.setIconImage(display.loadImage(iconBase + "-icon.png"));
						button.setDisabledImage(display.loadImage(iconBase + "-disabled.png"));
						button.setLitImage(display.loadImage(iconBase + "-lit.png"));
					}
					/* Try to set up the images */
					
					button.addSelectionListener(new SelectionAdapter()
					{
						public void widgetSelected(SelectionEvent e)
						{
							buttonClicked(button, current.getFunctionName(), id);
						}
					});
					/* Set up the SelectionListener */
				}
				else if(current instanceof SliderAdvertisement)
				{
					SliderAdvertisement currentAd = (SliderAdvertisement)current;
					final RemoteSlider slider = page.makeSlider(current.getFunctionName(), width, height);
					String helpText = help.getFunctionHelp(current.getFunctionName());
					if(helpText != null) slider.setHelpText(helpText);
					
					slider.getSlider().setMinimum(currentAd.getMin());
					slider.getSlider().setMaximum(currentAd.getMax());
					slider.setViewable(viewable);

					slider.addSelectionListener(new SelectionAdapter()
					{
						public void widgetSelected(SelectionEvent e)
						{
							sliderClicked(slider, current.getFunctionName(), id);
						}
					});
				}
				else if(current instanceof LabelAdvertisement)
				{
					RemoteLabel label = page.makeLabel(current.getFunctionName(), width, height);
					label.setViewable(viewable);
				}
				else if(current instanceof ImageAdvertisement)
				{
					RemoteImage image = page.makeImage(current.getFunctionName(), width, height);
				}
			}
		}
		
		display.resizeComponents();
		/* Resize the display */
		pageTable.put(ad.getDeviceID(), page);
		/* Store the page for later use */
	}
	
	/**
	 * Remove the specified device from the GUI.
	 *
	 * @param id the ID of the device to be removed
	 */
	public void deleteDevice(final String id)
	{
		waitForDisplay();
		display.runCommand(new Runnable()
		{
			public void run()
			{
				runDeleteDevice(id);
			}
		});
	}
	
	private void runDeleteDevice(String id)
	{
		RemotePage deleteMe = (RemotePage)pageTable.get(id);
		
		if(deleteMe != null)
		{
			savePage(id, deleteMe);
			display.removePage(deleteMe);
			pageTable.remove(id);
			/* If the page still exists, save it, then get rid of it */
		}
		display.resizeComponents();
	}
	
	private void sliderClicked(final RemoteSlider src, String functionName, final AgentID id)
	{
		display.runCommand(new Runnable()
		{
			public void run()
			{
				src.setHighlighted(!src.isHighlighted());
				/* Highlight the slider */
				
				new Timer().schedule(new TimerTask()
				{
					public void run()
					{
						display.runCommand(new Runnable()
						{
							public void run()
							{
								src.setHighlighted(!src.isHighlighted());
							}
						});
					}
				}, 150);
				/* Wait 150 ms, then de-highlight it */
			}
		});

		final int value = src.getSlider().getSelection();
		/* See what the slider's set to */
		
		new Thread()
		{
			public void run()
			{
				ActionInfo cmd = new ActionInfo(src.getFunctionName(), value);
				ActionTuple sendMe = new ActionTuple(cmd);
				sendMe.addField(new EField("aID", agent.getAgentID()));
				/* Make a corresponding ActionTuple */
				
				AcquaintanceList aql = agent.getAQL();
				AgentProxy proxy = aql.getAgentProxy(id);
				if(proxy == null) return;
				/* Try to get a proxy to the device */
				
				try
				{
					proxy.out(sendMe);
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
				/* Then try to out the ActionTuple into the agent's tuple space */
			}
		}.start();
	}

	private void buttonClicked(final RemoteButton src, String functionName, final AgentID id)
	{
		display.runCommand(new Runnable()
		{
			public void run()
			{
				src.setHighlighted(!src.isHighlighted());
				
				new Timer().schedule(new TimerTask()
				{
					public void run()
					{
						display.runCommand(new Runnable()
						{
							public void run()
							{
								src.setHighlighted(!src.isHighlighted());
							}
						});
					}
				}, 150);
			}
		});

		new Thread()
		{
			public void run()
			{
				ActionInfo cmd = new ActionInfo(src.getFunctionName(), !src.isLit());
				ActionTuple sendMe = new ActionTuple(cmd);
				sendMe.addField(new EField("aID", agent.getAgentID()));
				
				AcquaintanceList aql = agent.getAQL();
				AgentProxy proxy = aql.getAgentProxy(id);
				if(proxy == null) return;
				
				try
				{
					proxy.out(sendMe);
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
		}.start();
	}
	
	/**
	 * Update the GUI to reflect a device's new state.
	 *
	 * @param info the state information of the device to be updated
	 */
	public void update(final StateInfo info)
	{
		waitForDisplay();
		display.runCommand(new Runnable()
		{
			public void run()
			{
				runUpdate(info);
			}
		});
	}
	
	private void runUpdate(StateInfo info)
	{
		RemotePage page = (RemotePage)pageTable.get(info.getDeviceID());
		if(page == null) return;
		
		Hashtable stateTable = info.getFunctionToState();
		Vector kids = page.getPanel().getComponents();
		
		/* Look through all the components */
		for(Enumeration enum = kids.elements(); enum.hasMoreElements();)
		{
			RemoteComponent current = (RemoteComponent)enum.nextElement();
			if(current instanceof RemoteBlank) continue;
			/* There's nothing to be done for blanks */

			ComponentState currentState = (ComponentState)stateTable.get(current.getFunctionName());
			if(currentState == null) continue;
			/* See if our table of update states has this component */
						
			if(current instanceof RemoteButton)
			{
				RemoteButton button = (RemoteButton)current;
				ButtonState buttonState = (ButtonState)currentState;
				
				boolean enabled = buttonState.isEnabled();
				if(button.isClickable() != enabled)
					button.setClickable(enabled);
					
				boolean lit = buttonState.isLit();
				if(button.isLit() != lit)
					button.setLit(lit);
			}
			else if(current instanceof RemoteSlider)
			{
				RemoteSlider slider = (RemoteSlider)current;
				Slider child = slider.getSlider();
				SliderState sliderState = (SliderState)currentState;
				
				if(child.isFocusControl()) continue;

				boolean enabled = sliderState.isEnabled();
				if(slider.isClickable() != enabled)
					slider.setClickable(enabled);
					
				int value = sliderState.getValue();
				if(!slider.isEditing() && child.getSelection() != value)
					child.setSelection(value);
					
				String text = sliderState.getLabel();
				if(!text.equals(slider.getText()))
					slider.setText(text);
			}
			else if(current instanceof RemoteLabel)
			{
				RemoteLabel label = (RemoteLabel)current;
				LabelState labelState = (LabelState)currentState;

				boolean enabled = labelState.isEnabled();
				if(label.isClickable() != enabled)
					label.setClickable(enabled);

				String text = labelState.getText();
				if(!text.equals(label.getText()))
					label.setText(text);
			}
			else if(current instanceof RemoteImage)
			{
				RemoteImage image = (RemoteImage)current;
				ImageState imageState = (ImageState)currentState;
				
				java.awt.image.BufferedImage img = imageState.getImage();
				Image swtImg = RemoteIconManager.imageSWTtoAWT(image.getDisplay(), img);
				image.setImage(swtImg);
			}
			/* Read all the state values and change the component if necessary */
		}
	}
}
