package uremote.swt.ui;

import java.util.Enumeration;
import java.util.Vector;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.*;

/**
 * A common class that all components to be shown on the display should
 * extend.
 *
 * @author Greg Hackmann
 */
public abstract class RemoteComponent extends Canvas implements PaintListener, MouseListener
{
	private GridData data, newData;
	private Image normalImage, disabledImage, litImage, iconImage;
	private Vector clickListeners = new Vector();
	private SelectionListener editListener;
	private String text;
	private String helpText;
	private boolean clickable = true, lit = false, editing = false,
	                  viewable = true, highlighted = false, gridShowing = false;
	private int gridWidth, gridHeight;
	private String id;
	private ComponentResizer resizer;
	
	
	/**
	 * Instantiate the component to the default size.
	 *
	 * @param parent the Composite that will contain this component
	 * @param id the function name of the component
	 */
	public RemoteComponent(Composite parent, String id)
	{
		this(parent, id, RemoteDisplay.GRID_SIZE /*- 1*/, RemoteDisplay.GRID_SIZE);
	}
	
	/**
	 * Instantiate the component to the given size.
	 *
	 * @param parent the Composite that will contain this component
	 * @param id the function name of the component
	 * @param gridWidth the width of a single grid square, in pixels
	 * @param gridHeight the height of a single grid square, in pixels
	 */
	public RemoteComponent(Composite parent, String id, int gridWidth, int gridHeight)
	{
		super(parent, SWT.NONE);
		this.id = id;
		this.gridWidth = gridWidth;
		this.gridHeight = gridHeight;
		/* Save the passed parameters */
		
		data = new GridData(GridData.FILL_HORIZONTAL);
		newData = new GridData(GridData.FILL_HORIZONTAL);
		setLayoutData(data);
		setLayout(new FillLayout());
		/* Set up the layout */
		
		addDisposeListener(new DisposeListener()
		{
			public void widgetDisposed(DisposeEvent e)
			{
				if(normalImage != null) normalImage.dispose();
				if(disabledImage != null) disabledImage.dispose();
				if(litImage != null) litImage.dispose();
				if(iconImage != null) iconImage.dispose();
			}
		});
		/* Dispose all the Images once the RemoteComponent dies */
		
		addPaintListener(this);
		addMouseListener(this);
		/* We want to know when we're getting painted and who's clicking on us */
		
		resizer = new ComponentResizer(this, this);
		/* Listen for resizing */
	}
	
	protected void setClickSource(Control source)
	{
		resizer.setSource(source);
	}
	
	/**
	 * Set the component's "normal" image.  Not all components may use this image.
	 *
	 * @param i the image to use
	 */
	public void setImage(Image i) { normalImage = i; }
	
	/**
	 * Set the component's "disabled" image.  Not all components may use this image.
	 *
	 * @param i the image to use
	 */
	public void setDisabledImage(Image i) { disabledImage = i; }
	
	/**
	 * Get the component's "disabled" image.
	 *
	 * @return the component's "disabled" image
	 */
	public Image getDisabledImage() { return disabledImage; }

	/**
	 * Set the component's "lit" image.  Not all components may use this image.
	 *
	 * @param i the image to use
	 */
	public void setLitImage(Image i) { litImage = i; }
	
	/**
	 * Get the component's "lit" image.
	 *
	 * @return the component's "lit" image
	 */
	public Image getLitImage() { return litImage; }

	/**
	 * Set the component's "icon" image, which is shown on the help page.
	 * Not all components may use this image.
	 *
	 * @param i the image to use
	 */
	public void setIconImage(Image i) { iconImage = i; }
	
	/**
	 * Get the component's "icon" image, which is shown on the help page.
	 *
	 * @return the component's "icon" image
	 */
	public Image getIconImage() { return iconImage; }

	/**
	 * Get the component's current image.
	 *
	 * @return the component's current image
	 */
	public Image getImage()
	{
		/* In editing mode, non-viewable components should look disabled (not
		   invisible, since that makes it hard to edit).  Otherwise, the
		   component looks normal. */
		if(editing)
		{
			if(!viewable && disabledImage != null) return disabledImage;
			return normalImage;
		}
		
		/* If it's not clickable, try using the "disabled" image */
		if(!clickable && disabledImage != null) return disabledImage;
		/* If it's lit, try the "lit" image */
		if(lit && litImage != null) return litImage;
		/* The best we can do otherwise is the normal image */
		return normalImage;
	}
	
	/**
	 * Set the component's text.  Not all component may use this label.
	 *
	 * @param t the text to use
	 */
	public void setText(String t) { text = t; }
	
	/**
	 * Get the component's text.
	 *
	 * @return the component's text
	 */
	public String getText() {  return text; }
	
	/**
	 * Set the component's "help" text, which is shown on the help page.
	 *
	 * @param t the text to use
	 */
	public void setHelpText(String t) { helpText = t; }
	
	/**
	 * Get the component's "help" text.
	 *
	 * @return the component's text
	 */
	public String getHelpText() { return helpText; }
	
	/**
	 * Add a {@link org.eclipse.swt.events.SelectionListener} to the list of
	 * listeners.  This listener will be invoked when the component is
	 * clicked on.
	 *
	 * @param listener the listener interested in this component
	 */
	public void addSelectionListener(SelectionListener listener)
	{
		clickListeners.addElement(listener);
	}
	
	/**
	 * Set the {@link org.eclipse.swt.events.SelectionListener} for when the
	 * component is in "edit" mode.
	 *
	 * @param listener the listener interested in this component
	 */
	public void setEditListener(SelectionListener listener)
	{
		editListener = listener;
	}
	
	/**
	 * Set the size of the grid.  This should not normally be called by users.
	 *
	 * @param w the width of a grid square, in pixels
	 * @param h the height of a grid square, in pixels
	 */
	public void setGridSize(int w, int h)
	{
		gridWidth = w; gridHeight = h;
	}
	
	/**
	 * Compute the size of the component.  This is used by the SWT library and
	 * should not be called by users.
	 */
	public Point computeSize(int wHint, int hHint, boolean changed)
	{
		Point size = calcSize();
		if(wHint != SWT.DEFAULT) size.x = wHint;
		if(hHint != SWT.DEFAULT) size.y = hHint;
		/* Get the size, and override anything that's not hinted */
		
		return size;
	}
	
	/**
	 * Calculate the size of the component based on the grid size.  Note that
	 * this size may not necessarily match the component's on-screen size, since
	 * SWT may use "hints" to change the actual size of a component.
	 *
	 * @return the calculated size of the component
	 */
	public Point calcSize()
	{
		int width = data.horizontalSpan * (gridWidth - 1);
		int height = data.verticalSpan * gridHeight;
		
		return new Point(width, height);
	}
	
	/**
	 * Set the width of the component.  Note that this does not actually change
	 * the component's on-screen size until {@link #resize()} is called.
	 *
	 * @param w the width in grid squares
	 */
	public void setWidth(int w) { newData.horizontalSpan = w; }
	
	/**
	 * Get the width of the component.
	 *
	 * @return the width in grid squares
	 */
	public int getWidth() { return newData.horizontalSpan; }
	
	/**
	 * Set the height of the component.  Note that this does not actually change
	 * the component's on-screen size until {@link #resize()} is called.
	 *
	 * @param h the height in grid squares
	 */
	public void setHeight(int h) { newData.verticalSpan = h; }
	
	/**
	 * Get the height of the component.
	 *
	 * @return the height in grid squares
	 */
	public int getHeight() { return newData.verticalSpan; }

	/**
	 * Swap this component's on-screen location with that of another.
	 *
	 * @param other the component to swap with
	 */
	public void swap(RemoteComponent other)
	{
		Composite otherParent = other.getParent();
		GridData otherData = other.newData;
		
		other.setParent(getParent());
		setParent(otherParent);
		/* Swap parents to physically move them */
		
		other.newData = newData;
		newData = otherData;
		/* Swap the layouts */
		
		resize();
		other.resize();
		/* Then make the layouts current */
	}
		
	/**
	 * Set whether or not this component is enabled.  {@link #setEnabled(boolean)} is
	 * reserved for use by SWT and should not be used for this.
	 *
	 * @param flag true if enabled, false if disabled
	 */
	public void setClickable(boolean flag) { clickable = flag; redraw(); }
	/**
	 * Get whether or not this component is enabled.
	 *
	 * @return true if enabled, false if disabled
	 */
	public boolean isClickable() { return clickable; }
	
	/**
	 * Set whether or not this component is lit.
	 *
	 * @param flag true if lit, false if unlit
	 */
	public void setLit(boolean flag) { lit = flag; redraw(); }
	/**
	 * Get whether or not this component is lit.
	 *
	 * @return true if lit, false if unlit
	 */
	public boolean isLit() { return lit; }

	/**
	 * Set whether or not this component is in edit mode.
	 *
	 * @param flag true if in edit mode, false if not
	 */
	public void setEditing(boolean flag)
	{
		editing = flag;
		if(editing) setVisible(true);
		/* Components should never be invisible while editing (otherwise
		   it's sort of hard to edit them) */
		else
		{
			setVisible(viewable);
			highlighted = false;
		}
		/* When not editing, make sure they're not highlighted */

		redraw();
	}
	/**
	 * Get whether or not this component is in edit mode.
	 *
	 * @return true if in edit mode, false if not
	 */
	public boolean isEditing() { return editing; }

	/**
	 * Set whether or not this component is visible.  {@link #setVisible(boolean)}
	 * is reserved for use by SWT and should not be used for this.
	 *
	 * @param flag true if visible, false if invisible
	 */
	public void setViewable(boolean flag)
	{
		viewable = flag;
		if(editing) setVisible(true);
		else setVisible(flag);
		
		redraw();
	}
	/**
	 * Get whether or not this component is visible.
	 *
	 * @return true if visible, false if invisible
	 */
	public boolean isViewable() { return viewable; }

	/**
	 * Set whether or not this component is highighted.
	 *
	 * @param flag true if highlighted, false if not
	 */
	public void setHighlighted(boolean flag)
	{
		highlighted = flag;
		redraw();
	}

	/**
	 * Get whether or not this component is highlighted.
	 *
	 * @return true if highlighted, false if not
	 */
	public boolean isHighlighted() { return highlighted; }

	/**
	 * Get whether or not this component's grid square is visible.
	 *
	 * @param flag true if visible, false if invisible
	 */
	public void setGridVisible(boolean flag)
	{
		gridShowing = flag;
		if(!editing) redraw();
	}
	/**
	 * Get whether or not this component's grid square is visible.
	 *
	 * @return true if visible, false if invisible
	 */
	public boolean isGridShowing() { return gridShowing; }

	/**
	 * Paint the component-specific part of the component.  All classes that
	 * extend RemoteComponent must override this.  This is used by the SWT
	 * library and should not be called by users.
	 */
	public abstract void onPaint(PaintEvent e);
	
	/**
	 * Paint the component.  This is used by the SWT library and should not be
	 * called by users.
	 */
	public void paintControl(PaintEvent e)
	{
		onPaint(e);
		/* Draw the component-specific part so the rectangles are drawn on top */
		
		if(isHighlighted())
		{
			GC gc = e.gc;
			Color highlightColor = new Color(getDisplay(), 0, 0, 128);
			
			gc.setForeground(highlightColor);
			gc.setLineWidth(2);
			gc.drawRectangle(2, 1, getSize().x - 3, getSize().y - 3);
			
			highlightColor.dispose();
		}
		/* Draw the hightlight if it's highlighted */
		
		if(isEditing() || isGridShowing())
		{
			GC gc = e.gc;
		
			gc.setForeground(getParent().getForeground());
			gc.setLineWidth(1);
			gc.setLineStyle(isViewable() ? SWT.LINE_SOLID : SWT.LINE_DOT);
			gc.drawRectangle(0, 0, getSize().x - 1, getSize().y - 1);
		}
		/* Otherwise, if we need a border, draw one */
	}
	
	/**
	 * Respond to the mouse being clicked within this component.  This is used
	 * by the SWT library and should not be called by users.
	 */
	public void mouseDown(MouseEvent e)
	{
		Event evt = new Event();
		evt.widget = this;
		evt.x = e.x;
		evt.y = e.y;
		SelectionEvent selEvt = new SelectionEvent(evt);
		/* Make a SelectionEvent out of the MouseEvent */
		
		if(editing)
		{
		//	if(editListener != null) editListener.widgetSelected(selEvt);
			return;
		}
		/* If we're editing, let the ComponentResizer handle it */
		
		if(!clickable) return;
		/* If nobody can click it, forget it */

		for(Enumeration enum = clickListeners.elements(); enum.hasMoreElements();)
			((SelectionListener)enum.nextElement()).widgetSelected(selEvt);
		/* Otherwise, notify the listeners */

//		for(int i = 0; i < clickListeners.size(); i++)
//			((SelectionListener)clickListeners.elementAt(i)).widgetSelected(selEvt);
	}
	
	/**
	 * Respond to the mouse being double-clicked within this component.  This is
	 * used by the SWT library and should not be called by users.
	 */
	public void mouseDoubleClick(MouseEvent e) { }

	/**
	 * Respond to the mouse being released within this component.  This is
	 * used by the SWT library and should not be called by users.
	 */
	public void mouseUp(MouseEvent e) { }
	
	/**
	 * Resize the component on-screen after changes to its size.
	 */
	public void resize()
	{
		data.horizontalSpan = newData.horizontalSpan;
		data.verticalSpan = newData.verticalSpan;
		/* Use the new layout */
		
		setSize(calcSize());
		/* Resize the component */
		getParent().setSize(getParent().computeSize(SWT.DEFAULT, SWT.DEFAULT));
		/* Then resize its parent */

		redraw();
	}
	
	/**
	 * Return the function name (ID) of this component.
	 *
	 * @return the function name of the current component
	 */
	public String getFunctionName() { return id; }
	
	private class ComponentResizer implements MouseListener, MouseMoveListener
	{
		private Control source;
		private RemoteComponent target;
		private static final int THRESHOLD = 5;
		private int currentCursor = SWT.CURSOR_ARROW;
		private boolean dragging = false;
	
		public ComponentResizer(Composite source, RemoteComponent target)
		{
			this.source = source;
			this.target = target;
			source.addMouseListener(this);
			source.addMouseMoveListener(this);
			/* Tell the source this class wants to listen in */
		}
	
		public void setSource(Control newSource)
		{
			source.removeMouseListener(this);
			source.removeMouseMoveListener(this);
			source = newSource;
			if(source != null)
			{
				source.addMouseListener(this);
				source.addMouseMoveListener(this);
			} 
			/* Transfer the listeners to the new source */
		}
	
		public void mouseMove(MouseEvent evt)
		{
			/* If the mouse is already down . . . */
			if(dragging)
			{
			
		//		if(currentCursor == SWT.CURSOR_ARROW) return;

				boolean changed = false;
				boolean bottom = (currentCursor == SWT.CURSOR_SIZES || currentCursor == SWT.CURSOR_SIZESE);
				boolean right = (currentCursor == SWT.CURSOR_SIZEE || currentCursor == SWT.CURSOR_SIZESE);
				/* Use the type of cursor to figure out what direction(s) we're resizing */

				if(bottom)
				{
					int y = evt.y;
					if(y > source.getSize().y + 15)
					{
						target.setHeight(target.getHeight() + 1);
						changed = true;
					}
					else if(y < source.getSize().y - 15)
					{
						target.setHeight(target.getHeight() - 1);
						changed = true;
					}
					/* Make the component one taller or one shorter if the drag
					   strayed far enough from the component's bottom border */
				}
				if(right)
				{
					int x = evt.x;
					if(x > source.getSize().x + 15)
					{
						target.setWidth(target.getWidth() + 1);
						changed = true;
					}
					else if(x < source.getSize().x - 15)
					{
						target.setWidth(target.getWidth() - 1);
						changed = true;
					}
					/* Now do the same for the right border */
				}

				if(changed)
				{
					target.resize();
					mouseUp(evt);
					/* If anything changed, update the component, then force
					   the drag to end.  (If we don't force the drag to end,
					   then the component tends to osciallate between two
					   different sizes until the user lets go.) */
				}
				
				return;
			}
		
			boolean s = south(evt), e = east(evt);
			int cursor = SWT.CURSOR_ARROW;
			/* See if the mouse is over any borders */
		
			if(!editing) cursor = SWT.CURSOR_ARROW;
			else if(s && e) cursor = SWT.CURSOR_SIZESE;
			else if(s) cursor = SWT.CURSOR_SIZES;
			else if(e) cursor = SWT.CURSOR_SIZEE;
			/* If it is, get the appropriate cursor */
		
			getShell().setCursor(new Cursor(getDisplay(), cursor));
			currentCursor = cursor;
			/* Update the cursor */
		}
	
		public void mouseUp(MouseEvent evt)
		{
			dragging = false;
			/* We're done dragging */
		}
	
		private boolean south(MouseEvent e)
		{
			return (e.y > source.getSize().y - THRESHOLD);
		}
	
		private boolean east(MouseEvent e)
		{
			return (e.x > source.getSize().x - THRESHOLD);
		}

		public void mouseDoubleClick(MouseEvent evt)
		{
			if(editing) target.setViewable(!target.isViewable());
			/* If we're in editing mode, flip the target's visibility */
		}

		public void mouseDown(MouseEvent evt)
		{
			dragging = true;
			/* We've started dragging */
			
			/* If we're not resizing . . . */
			if(currentCursor == SWT.CURSOR_ARROW)
			{
				Event evt2 = new Event();
				evt2.widget = target;
				evt2.x = evt.x;
				evt2.y = evt.y;
				SelectionEvent selEvt = new SelectionEvent(evt2);
				/* Make a SelectionEvent out of the MouseEvent */
		
				if(editing && editListener != null)
					editListener.widgetSelected(selEvt);
				/* Notify the edit listener if necessary */
			}
		}
	}
}