package uremote.swt.ui;

import uremote.*;

import java.util.Hashtable;
import java.util.Vector;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.custom.*;

/**
 * A class representing the Universal Remote's GUI.
 * Note that due to limitations in SWT, the RemoteDisplay must be instantiated
 * in the same thread that calls {@link #open()}, and any alterations to the
 * UI must be done with {@link Runnable}s passed to {@link #runCommand(Runnable)}.
 *
 * @author Greg Hackmann
 */
public class RemoteDisplay
{
	private Display display;
	private Shell shell, messageShell;
	
	private CTabFolder tabs;
	private RemotePanel buttonPanel;
	
	private Vector pages = new Vector(4);
	private boolean helpShowing = false, editing = false, gridShowing = false, noDeviceShowing = false;;
	private RemotePage helpDevice, helpMain;
	private Image helpImage, warningImage;
	
	private RemoteButton upButton, downButton, helpButton, gridButton, editButton;
	private int currentPage = 0;
	
	private Color background, foreground;
	
	private Runnable closingRunnable = null;
	private Hashtable images;
	
	private RemotePage noDevicePage = null;
	
	final static int GRID_SIZE = 48;
	final static int NUM_COLUMNS = 5;
	
	/**
	 * Instantiate, but do not show, the GUI.  The GUI should be shown using
	 * the {@link #open()} method.
	 */
	public RemoteDisplay()
	{
		display = new Display();
		shell = new Shell(display, SWT.DIALOG_TRIM);
		shell.setText("Universal Remote");
		/* Set up the dialog box */
				
		background = new Color(display, 178, 178, 178);
//		foreground = new Color(display, 26, 26, 26);
		shell.setBackground(background);
//		shell.setForeground(foreground);
		/* Color it */
		
		GridLayout layout = new GridLayout();
		layout.horizontalSpacing = layout.verticalSpacing = layout.marginWidth = layout.marginHeight = 0;
		layout.numColumns = 1;
		shell.setLayout(layout);
		/* Make a GridLayout for the upper and lower parts */
		
		Composite tabPanel = new Composite(shell, SWT.NONE);
		tabPanel.setBackground(shell.getBackground());
		tabPanel.setForeground(shell.getForeground());
		tabPanel.setLayout(new FillLayout(SWT.VERTICAL));
		tabPanel.setLayoutData(new GridData(GridData.FILL_BOTH));
		/* Make the upper part for the tabs */
				
		tabs = new CTabFolder(tabPanel, SWT.BORDER | SWT.FLAT);
		tabs.setSelectionBackground(new Color [] { new Color(display, 219, 219, 219), background }, new int [] { 100 });
		tabs.setBackground(shell.getBackground());
		tabs.setForeground(shell.getForeground());
		/* Now make the tabs */
		
		tabs.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				resizeComponents();
			}
		});
		/* We want to know when somebody clicks on a tab */
		
		buttonPanel = new RemotePanel(shell, GRID_SIZE, 42);
		buttonPanel.setLayoutData(new GridData(GridData.FILL_BOTH));
		GridData data = new GridData();
		data.verticalAlignment = GridData.VERTICAL_ALIGN_END;
		buttonPanel.setLayoutData(data);
		/* Make the lower part for the "global" buttons */
		
		images = new Hashtable();
		
		upButton = buttonPanel.makeButton("_scroll_up", 1, 1);
		upButton.setImage(loadImage(URemoteConstants.path + "pics/buttons/scroll-up.png"));
		upButton.setDisabledImage(loadImage(URemoteConstants.path + "pics/buttons/blank.png"));
		
		downButton = buttonPanel.makeButton("_scroll_down", 1, 1);
		downButton.setImage(loadImage(URemoteConstants.path + "pics/buttons/scroll-down.png"));
		downButton.setDisabledImage(loadImage(URemoteConstants.path + "pics/buttons/blank.png"));
		/* Make scroll buttons */
		
		gridButton = buttonPanel.makeButton("_toggle_grid", 1, 1);
		gridButton.setImage(loadImage(URemoteConstants.path + "pics/buttons/grid.png"));
		gridButton.setLitImage(loadImage(URemoteConstants.path + "pics/buttons/grid-lit.png"));
		gridButton.setDisabledImage(loadImage(URemoteConstants.path + "pics/buttons/grid-disabled.png"));
		/* Make a grid toggle button */
		
		editButton = buttonPanel.makeButton("_toggle_edit", 1, 1);
		editButton.setImage(loadImage(URemoteConstants.path + "pics/buttons/edit.png"));
		editButton.setLitImage(loadImage(URemoteConstants.path + "pics/buttons/edit-lit.png"));
		editButton.setDisabledImage(loadImage(URemoteConstants.path + "pics/buttons/edit-disabled.png"));
		/* Make an edit toggle button */
		
		helpButton = buttonPanel.makeButton("_toggle_help", 1, 1);
		helpButton.setImage(loadImage(URemoteConstants.path + "pics/buttons/help.png"));
		helpButton.setLitImage(loadImage(URemoteConstants.path + "pics/buttons/help-lit.png"));
		helpButton.setDisabledImage(loadImage(URemoteConstants.path + "pics/buttons/help-disabled.png"));
		/* Make a help toggle button */
		
		upButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				scroll(GRID_SIZE * 3);
			}
		});
		
		downButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				scroll(-GRID_SIZE * 3);
			}
		});
		
		gridButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				toggleGrid();
			}
		});
		
		helpButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				toggleHelp();
			}
		});
		
		editButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				toggleEdit();
			}
		});
		/* Set up listeners for the buttons */
				
		helpImage = loadImage(URemoteConstants.path + "pics/icons/help.png");
		warningImage = loadImage(URemoteConstants.path + "pics/icons/warning.png");
		/* Save the help image for later so we don't have to constantly reload it */
	}
	
	private void scroll(int delta)
	{
		Composite scrollMe = ((RemotePanel)tabs.getSelection().getControl()).getContents();
		/* Find out what we're supposed to scroll */
		
		int x = scrollMe.getLocation().x;
		int y = scrollMe.getLocation().y + delta;
		/* Move it by the appropriate amount */
		
		if (y > 0 || y < -scrollMe.getSize().y) return;
		/* Don't bother if we can't move */
		
		scrollMe.setLocation(x, y);
		updateButtons();
		/* Otherwise scroll and update the buttons */
	}
	
	private void updateButtons()
	{
		RemotePanel selected = null;
		if(tabs.getSelection() != null) selected = (RemotePanel)tabs.getSelection().getControl();

		if(selected == null)
		{
			upButton.setClickable(false);
			downButton.setClickable(false);
			editButton.setClickable(false);
			gridButton.setClickable(false);
			helpButton.setClickable(false);
			return;
		}
		/* If no tab is visible, forget it */
		
		Composite scrollMe = selected.getContents();
		upButton.setClickable(scrollMe.getLocation().y != 0);
		downButton.setClickable(scrollMe.getSize().y + scrollMe.getLocation().y > GRID_SIZE * 4);
		editButton.setClickable(!helpShowing && !noDeviceShowing);
		gridButton.setClickable(!helpShowing && !noDeviceShowing && !editing);
		helpButton.setClickable(!noDeviceShowing);
		/* Set the buttons based on whether or not there's space to move in
		   each direction */
	}
	
	/**
	 * Resize the components and update the window.  This should not normally
	 * be called by users.
	 */
	public void resizeComponents()
	{
		RemotePanel selected = null;
		if(tabs.getSelection() != null) selected = (RemotePanel)tabs.getSelection().getControl();
		if(selected == null)
		{
			upButton.setClickable(false);
			downButton.setClickable(false);
			editButton.setClickable(false);
			gridButton.setClickable(false);
			helpButton.setClickable(false);
			return;
		};
		/* If no tab is visible, forget it */
		
		Vector kids = selected.getComponents();
		for(int i = 0; i < kids.size(); i++)
		{
			RemoteComponent current = (RemoteComponent)kids.elementAt(i);
			if(!(current instanceof RemoteBlank)) current.resize();
			current.setGridVisible(gridShowing);
			/* Resize everything on the tab that's not a blank */
		}
		
//		selected.setGridVisible(gridShowing);
		updateButtons();
		/* We may need to change the scroll buttons now */
	}

	private void toggleHelp()
	{
		/* If the help page is already up . . . */
		if(helpShowing)
		{
			for(int i = 0; i < pages.size(); i++)
			{
				RemotePage current = (RemotePage)pages.elementAt(i);
				current.show();
			}
			if(noDevicePage != null) noDevicePage.show();
			/* Put all the old pages back */
			
			tabs.setSelection(currentPage + ((helpMain == null || noDevicePage != null) ? 1 : 2));
			/* Go back to the old page */
			
			if(helpDevice != null) helpDevice.dispose();
			if(helpMain != null) helpMain.dispose();
			/* Get rid of the help pages */
		}
		else
		{
			currentPage = tabs.getSelectionIndex();
			/* Remember which tab is selected */
			helpDevice = helpMain = null;
			
			RemotePanel helpPanel;

			CTabItem helpTab = new CTabItem(tabs, SWT.NONE);
			RemotePage makeMyPage = (RemotePage)pages.elementAt(currentPage);
			/* Get the corresponding page */

			if(editing) helpPanel = RemoteHelp.makeEditHelp(tabs, this);
			else helpPanel = makeMyPage.makeHelpPage();
			/* Make the edit or context-sensitive help */
			helpTab.setControl(helpPanel);
			/* Put it in the tab */
			
			helpDevice = new RemotePage(helpPanel, helpTab);
			helpDevice.setText(editing ? "Editing Help" : makeMyPage.getTab().getText());
			helpDevice.setImage(helpImage);
			/* Now put the tab in a page */

			if(!editing)
			{
				helpPanel = RemoteHelp.makeMainHelp(tabs, this);
				helpTab = new CTabItem(tabs, SWT.NONE);
				helpTab.setControl(helpPanel);
				
				helpMain = new RemotePage(helpPanel, helpTab);
				helpMain.setText("Help");
				helpMain.setImage(helpImage);
			}
			/* If we're not editing, then we want the main help, too */
			else helpMain = null;
			
			for(int i = 0; i < pages.size(); i++)
			{
				RemotePage current = (RemotePage)pages.elementAt(i);
				current.hide();
			}
			/* Hide all the device pages */
		}
		
		helpShowing = !helpShowing;
		editButton.setClickable(!helpShowing);
		gridButton.setClickable(!helpShowing && !editing);
		helpButton.setLit(helpShowing);
		resizeComponents();
		/* Update the state and buttons */
	}
	
	private void toggleEdit()
	{
		editing = !editing;
		
		RemotePanel editMe = (RemotePanel)tabs.getSelection().getControl();
		editMe.setEditing(editing);
		/* Tell the panel about the change */
		
		gridButton.setClickable(!editing);
		editButton.setLit(editing);
		/* Then update the buttons */
	}
	
	private void toggleGrid()
	{
		gridShowing = !gridShowing;
		/* Flip whether or not the grid is showing */
		
		RemotePanel changeMe = (RemotePanel)tabs.getSelection().getControl();
		changeMe.setGridVisible(gridShowing);
		/* Tell the current tab about the change */
		
		gridButton.setLit(gridShowing);
		/* Light or dim the grid button */
	}
	
	private void showNoDevicePage()
	{
		CTabItem noDeviceTab = new CTabItem(tabs, SWT.NONE);
		RemotePanel noDevicePanel = RemoteHelp.makeNoDevicePage(tabs, this);
		/* Make the "no device" page */
		
		noDeviceTab.setControl(noDevicePanel);
		noDevicePage = new RemotePage(noDevicePanel, noDeviceTab);
		noDevicePage.setText("No Devices In Range");
		noDevicePage.setImage(warningImage);
		/* Put it in a tab */
		tabs.setSelection(noDeviceTab);
		
		noDeviceShowing = true;
	}
	
	/**
	 * Get the {@link org.eclipse.swt.widgets.Shell} corresponding to
	 * the GUI.
	 *
	 * @return the Shell corresponding to the GUI
	 */
	public Shell getShell()
	{
		return shell;
	}
	
	/**
	 * Create a {@link RemotePage} and add it to the tabs.
	 *
	 * @return the created RemotePage
	 */
	public RemotePage makePage()
	{
		return makePage(GRID_SIZE /*- 1*/, GRID_SIZE);
	}
	
	/**
	 * Create a {@link RemotePage} with a non-default grid size and add it to
	 * the tabs.
	 *
	 * @param gridWidth the width (in pixels) of each grid square
	 * @param gridHeight the height (in pixels) of each grid square
	 * @return the created RemotePage
	 */
	public RemotePage makePage(int gridWidth, int gridHeight)
	{
		CTabItem tab = new CTabItem(tabs, SWT.NONE);
		RemotePanel panel = new RemotePanel(tabs, gridWidth, gridHeight);
		tab.setControl(panel);
		/* Get a tab and a panel, and assemble them */
		
		RemotePage returnMe = new RemotePage(panel, tab);
//		returnMe.hide();
		pages.addElement(returnMe);
		updateButtons();
		/* Now put both in a page */

		if(noDeviceShowing)
		{
			noDevicePage.getTab().dispose();
			noDevicePage = null;
			noDeviceShowing = false;
		}
		/* Get rid of the "no device" page if it's there */
		
		if(helpShowing) returnMe.hide();
		/* Don't show the page if we're in help mode */
		else if(tabs.getSelection() == null) tabs.setSelection(tab);
		
		return returnMe;
	}
	
	/**
	 * Removes the specified RemotePage from the tabs.
	 *
	 * @param page the page to be removed
	 */
	public void removePage(RemotePage page)
	{
		if(pages.size() == 1) showNoDevicePage();
		/* If there's only 1 page left, we're about to have none, so show
		   the "no device" page */

		pages.removeElement(page);
		page.getTab().dispose();
		/* Get rid of the page and dispose its tab */
	}
	
	
	/**
	 * Shows the GUI display.  This method blocks until the program has
	 * terminated.
	 */
	public void open()
	{
		showNoDevicePage();
		/* When we start up, there are no devices in range */
								   
		if(URemoteConstants.isCE) shell.setMaximized(true);
		else shell.setSize(248, 294);
		shell.open();
		resizeComponents();
		/* Size and open the shell */
		
		while(!shell.isDisposed())
		{
			if(!display.readAndDispatch())
				display.sleep();
		}
		/* Main event loop */
		
		if(closingRunnable != null) closingRunnable.run();
		/* If anybody cares about the shell being closed, tell them now */
	}
	
	/**
	 * Determine whether or not the GUI is open.
	 *
	 * @return true if the GUI is open, false if not
	 */
	public boolean isOpen() { return !shell.isDisposed(); }
	
	/**
	 * A helper method to load an {@link org.eclipse.swt.graphics.Image},
	 * given a filename.
	 *
	 * @param filename the filename of the image to be loaded
	 * @return the Image corresponding to the given filename
	 */
	public Image loadImage(String filename)
	{
		Image returnMe = (Image)images.get(filename);
		/* First look it up in the cache */
		if(returnMe == null)
		{
			returnMe = new Image(display, filename);
			images.put(filename, returnMe);
		}
		/* Failing that, load it, then put it in the cache */
		
		return returnMe;
	}
	
	/**
	 * Run the given Runnable within the UI's event queue.  Any alterations
	 * to the UI must be done through this method.
	 *
	 * @param runMe the Runnable to run
	 */
	public void runCommand(Runnable runMe)
	{
		display.asyncExec(runMe);
	}
	
	/**
	 * Run the given Runnable when the UI has terminated.
	 *
	 * @param runMe the Runnable to run
	 */
	public void onClose(Runnable runMe)
	{
		closingRunnable = runMe;
	}
}
