package uremote.swt.ui;

import java.util.Hashtable;
import java.awt.image.*;
import org.eclipse.swt.graphics.*;
import uremote.URemoteConstants;

/**
 * A class to manage the icons supported by the Universal Remote.
 *
 * @author Greg Hackmann
 */
public abstract class RemoteIconManager
{
	private static final Hashtable buttonMap;
	private static final Hashtable deviceMap;
	
	static
	{
		buttonMap = new Hashtable();
		buttonMap.put("POWER", URemoteConstants.path + "pics/buttons/power");
		buttonMap.put("MUTE", URemoteConstants.path + "pics/buttons/mute");
		buttonMap.put("PLAY", URemoteConstants.path + "pics/buttons/play");
		buttonMap.put("STOP", URemoteConstants.path + "pics/buttons/stop");
		buttonMap.put("PAUSE", URemoteConstants.path + "pics/buttons/pause");
		buttonMap.put("RECORD", URemoteConstants.path + "pics/buttons/record");
		buttonMap.put("FASTFORWARD", URemoteConstants.path + "pics/buttons/ff");
		buttonMap.put("REWIND", URemoteConstants.path + "pics/buttons/rewind");
		buttonMap.put("PREVIOUS", URemoteConstants.path + "pics/buttons/previous");
		buttonMap.put("NEXT", URemoteConstants.path + "pics/buttons/next");
		buttonMap.put("ARROWLEFT", URemoteConstants.path + "pics/buttons/arrow-left");
		buttonMap.put("ARROWRIGHT", URemoteConstants.path + "pics/buttons/arrow-right");
		buttonMap.put("ARROWUP", URemoteConstants.path + "pics/buttons/arrow-up");
		buttonMap.put("ARROWDOWN", URemoteConstants.path + "pics/buttons/arrow-down");
		buttonMap.put("ARROWENTER", URemoteConstants.path + "pics/buttons/arrow-enter");
		
		deviceMap = new Hashtable();
		deviceMap.put("CD", URemoteConstants.path + "pics/icons/cd.png");
		deviceMap.put("COFFEE", URemoteConstants.path + "pics/icons/coffee.png");
		deviceMap.put("COMPUTER", URemoteConstants.path + "pics/icons/computer.png");
		deviceMap.put("DVD", URemoteConstants.path + "pics/icons/dvd.png");
		deviceMap.put("PRESENTATION", URemoteConstants.path + "pics/icons/presentation.png");
		deviceMap.put("PRINTER", URemoteConstants.path + "pics/icons/printer.png");
		deviceMap.put("RADIO", URemoteConstants.path + "pics/icons/radio.png");
		deviceMap.put("SLIDESHOW", URemoteConstants.path + "pics/icons/slide-show.png");
		deviceMap.put("TV", URemoteConstants.path + "pics/icons/tv.png");
		deviceMap.put("VCR", URemoteConstants.path + "pics/icons/vcr.png");
		deviceMap.put("X10", URemoteConstants.path + "pics/icons/light-bulb.png");
	}
	
	private RemoteIconManager() { }
	
	/**
	 * Get the "base" filename for the provided icon.  Note that this does
	 * not include the ".png" file extension, so suffixes like "-lit.png"
	 * can be attached.
	 * <p>
	 * Currently, the following icons are guaranteed to be supported:
	 * "ARROWDOWN", "ARROWENTER", "ARROWLEFT", "ARROWRIGHT", "ARROWUP",
	 * "FASTFORWARD", "MUTE", "NEXT", "PAUSE", "PLAY", "POWER", "PREVIOUS",
	 * "RECORD", "REWIND", and "STOP".
	 *
	 * @param pic the icon to look up
	 * @return the icon's "base" filename, or null if the icon is undefined
	 */
	public synchronized static String iconFilename(String pic)
	{
		return (String)buttonMap.get(pic);
	}
	
	/**
	 * Get the full filename for the provided device icon.
	 * <p>
	 * Currently, the following icons are guaranteed to be supported:
	 * "CD", "COFFEE", "COMPUTER", "DVD", "PRESENTATION", "PRINTER", "RADIO",
	 * "SLIDESHOW", "TV", "VCR", and "X10".
	 *
	 * @param pic the device icon to look up
	 * @return the icon's full filename, or null if the icon is undefined
	 */
	public synchronized static String deviceFilename(String pic)
	{
		return (String)deviceMap.get(pic);
	}
		
	/**
	 * Convert an AWT {@link java.awt.image.BufferedImage} to an SWT
	 * {@link org.eclipse.swt.graphics.Image}.
	 *
	 * @param device the device that will display the SWT image
	 * @param awtImg the AWT image to convert
	 * @return the image in the form of an SWT image
	 */
	public static Image imageSWTtoAWT(Device device, BufferedImage awtImg)
	{
		int width = awtImg.getWidth(), height = awtImg.getHeight();
		int [] data = new int[width * height];
		awtImg.getRGB(0, 0, width, height, data, 0, width);
		/* Get the AWT image's RGB data */
		
		PaletteData palette = new PaletteData(0xFF, 0xFF00, 0xFF0000);
		byte [] byteData = new byte[data.length * 3];
		/* Make a palette mask for the SWT image, and make a buffer to
		   put the data in */
		
		for(int i = 0; i < data.length; i++)
		{
			byteData[i * 3] = (byte)(data[i] & 0xFF);
			byteData[i * 3 + 1] = (byte)((data[i] & 0xFF00) >> 8);
			byteData[i * 3 + 2] = (byte)((data[i] & 0xFF0000) >> 16);
			/* Split each ABGR int into R, G, and B bytes, and store them
			   in the buffer */
		}
		
		ImageData swtData = new ImageData(width, height, 24, palette, 4, byteData);
		return new Image(device, swtData);
		/* Make an SWT Image out of the buffer */
	}
}
