package uremote.swt.ui;

import org.eclipse.swt.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.custom.*;

/**
 * A class representing a page to be placed in a tabbed list.
 *
 * @author Greg Hackmann
 */
public class RemotePage
{
	private RemotePanel panel;
	private CTabItem tab;
	private CTabFolder parent;
	private String label;
	private Image image;
	
	/**
	 * Instatitate a RemotePage from a given {@link RemotePanel}
	 * and {@link org.eclipse.swt.widgets.CTabItem}.  This method is
	 * normally not used; the {@link RemoteDisplay#makePage()} method should be
	 * used instead.
	 *
	 * @param panel the panel to add to the display
	 * @param tab the CTabItem that will contain the panel
	 */
	public RemotePage(RemotePanel panel, CTabItem tab)
	{
		this.panel = panel;
		this.tab = tab;
		parent = tab.getParent();
	}
	
	/**
	 * Get the {@link CTabItem} containing this panel.
	 *
	 * @return the CTabItem belonging to this page
	 */
	public CTabItem getTab() { return tab; }
	
	/**
	 * Get the {@link RemotePanel} contained by this page.
	 *
	 * @return the RemotePanel belonging to this page
	 */
	public RemotePanel getPanel() { return panel; }
	
	/**
	 * Set the tab's label.
	 *
	 * @param text the text to set as the label
	 */
	public void setText(String text) { label = text; if(tab != null) tab.setText(text); }
	
	/**
	 * Set the tab's icon.
	 *
	 * @param img the image to set as the icon
	 */
	public void setImage(Image img) { image = img; if(tab != null) tab.setImage(img); }


	/**
	 * Create a new 1 x 1 {@link RemoteBlank}.
	 *
	 * @return the created RemoteBlank
	 */
	public RemoteBlank makeBlank()
	{
		return makeBlank(1, 1);
	}

	/**
	 * Create a new {@link RemoteBlank} of the given size.
	 *
	 * @param w the width of the blank, in grid squares
	 * @param h the height of the blank, in grid squares
	 * @return the created RemoteBlank
	 */
	public RemoteBlank makeBlank(int w, int h)
	{
		return panel.makeBlank(w, h);
	}

	/**
	 * Create a new 5 x 1 {@link RemoteSlider}.
	 *
	 * @param id the function name of the slider
	 * @return the created RemoteSlider
	 */
	public RemoteSlider makeSlider(String id)
	{
		return makeSlider(id, 5, 1);
	}

	/**
	 * Create a new {@link RemoteSlider} of the given size.
	 *
	 * @param id the function name of the slider
	 * @param w the width of the slider, in grid squares
	 * @param h the height of the slider, in grid squares
	 * @return the created RemoteSlider
	 */
	public RemoteSlider makeSlider(String id, int w, int h)
	{
		return panel.makeSlider(id, w, h);
	}
		
	/**
	 * Create a new 1 x 1 {@link RemoteButton}.
	 *
	 * @param id the function name of the button
	 * @return the created RemoteButton
	 */
	public RemoteButton makeButton(String id)
	{
		return makeButton(id, 1, 1);
	}
	
	/**
	 * Create a new {@link RemoteButton} of the given size.
	 *
	 * @param id the function name of the button
	 * @param w the width of the button, in grid squares
	 * @param h the height of the button, in grid squares
	 * @return the created RemoteButton
	 */
	public RemoteButton makeButton(String id, int w, int h)
	{
		return panel.makeButton(id, w, h);
	}
	
	/**
	 * Create a new 1 x 1 {@link RemoteLabel}.
	 *
	 * @param id the function name of the label
	 * @return the created RemoteLabel
	 */
	public RemoteLabel makeLabel(String id)
	{
		return makeLabel(id, 1, 1);
	}
	
	/**
	 * Create a new {@link RemoteLabel} of the given size.
	 *
	 * @param id the function name of the label
	 * @param w the width of the label, in grid squares
	 * @param h the height of the label, in grid squares
	 * @return the created RemoteLabel
	 */
	public RemoteLabel makeLabel(String id, int w, int h)
	{
		return panel.makeLabel(id, w, h);
	}
	
	/**
	 * Create a new 1 x 1 {@link RemoteImage}.
	 *
	 * @param id the function name of the image
	 * @return the created RemoteImage
	 */
	public RemoteImage makeImage(String id)
	{
		return makeImage(id, 1, 1);
	}
	
	/**
	 * Create a new {@link RemoteImage} of the given size.
	 *
	 * @param id the function name of the image
	 * @param w the width of the image, in grid squares
	 * @param h the height of the image, in grid squares
	 * @return the created RemoteImage
	 */
	public RemoteImage makeImage(String id, int w, int h)
	{
		return panel.makeImage(id, w, h);
	}

	/**
	 * Make the tab invisible.
	 */
	public void hide()
	{
		if(tab == null) return;
		
		tab.setControl(null);
		tab.dispose();
		tab = null;
	}
	
	/**
	 * Make the tab visible.
	 */
	public void show()
	{
		if(tab != null) return;

		tab = new CTabItem(parent, SWT.BORDER);
		
		tab.setControl(panel);
		if(label != null) tab.setText(label);
		if(image != null) tab.setImage(image);
	}
		
	/**
	 * Change the tab's visibility.
	 *
	 * @param flag true to make the tab visible, false to make it invisible
	 */
	public void show(boolean flag) { if(flag) show(); else hide(); }
	
	/**
	 * Dispose the tab.  This should not normally be called by the user.
	 */
	public void dispose()
	{
		if(tab != null) tab.dispose();
		panel.dispose();
	}

	/**
	 * Determine whether or not the tab is showing.
	 *
	 * @return true if the tab is visible, false if it is invisible
	 */
	public boolean isShowing() { return tab != null; }
	
	/**
	 * Generate a second RemotePanel containing help text for all of the
	 * components contained in the current panel.
	 *
	 * @return the created page
	 */
	public RemotePanel makeHelpPage()
	{
		return RemoteHelp.makePageHelp(parent, this);
	}
}
