package uremote.swt.ui;

import java.util.*;
import org.eclipse.swt.*;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;

import uremote.PersistentLayout;

/**
 * A class representing a panel contained by a tabbed page.
 *
 * @author Greg Hackmann
 */
public class RemotePanel extends ScrolledComposite implements KeyListener
{
	private GridLayout layout;
	private Composite parent;
	private Composite child;
	private Vector children = new Vector(4);
	private Vector rows = new Vector(4);
	private int gridWidth, gridHeight;
	private boolean editing = false;
	private RemoteComponent firstClicked = null;
	
	private final static int EDGE_THRESHOLD = 5;
	
	/**
	 * Instantiate a RemotePanel with the default grid size.  This method is
	 * normally not used; the {@link RemoteDisplay#makePage()} method should be
	 * used to make a {@link RemotePage} instead, and then the
	 * {@link RemotePage#getPanel()} method should be used to get a RemotePanel.
	 *
	 * @param parent the Composite that will contain this panel
	 */
	public RemotePanel(Composite parent)
	{
		this(parent, RemoteDisplay.GRID_SIZE /*- 1*/, RemoteDisplay.GRID_SIZE);
	}
	
	/**
	 * Instantiate a RemotePanel with the given grid size.  This method is
	 * normally not used; the {@link RemoteDisplay#makePage()} method should be
	 * used to make a {@link RemotePage} instead, and then the
	 * {@link RemotePage#getPanel()} method should be used to get a RemotePanel.
	 *
	 * @param parent the Composite that will contain this panel
	 * @param gridWidth the width of a grid square, in pixels
	 * @param gridHeight the height of a grid square, in pixels
	 */
	public RemotePanel(Composite parent, int gridWidth, int gridHeight)
	{
		super(parent, SWT.NONE);
		this.parent = parent;
		this.gridWidth = gridWidth;
		this.gridHeight = gridHeight;
		
		setLayout(new FillLayout());
		child = new Composite(this, SWT.NONE);
		setContent(child);
		
		setBackground(parent.getBackground());
		setForeground(parent.getForeground());
		child.setBackground(parent.getBackground());
		child.setForeground(parent.getForeground());
		
		layout = new GridLayout();
		layout.horizontalSpacing = layout.verticalSpacing = layout.marginWidth = layout.marginHeight = 0;
		layout.numColumns = RemoteDisplay.NUM_COLUMNS;
		child.setLayout(layout);
	}
	
	private Composite makeParent()
	{
		Composite myParent = new Composite(child, SWT.NONE);
		myParent.setLayout(new FillLayout());
		myParent.setBackground(getBackground());
		myParent.setForeground(getForeground());
		
		return myParent;
	}
	
	private void initComponent(RemoteComponent initMe, int w, int h)
	{
		initMe.setWidth(w);
		initMe.setHeight(h);
		initMe.getParent().setLayoutData(initMe.getLayoutData());
		initMe.setEditListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				editClicked(e);
			}
		});
		initMe.addKeyListener(this);
		
		children.addElement(initMe);
		child.setSize(child.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}
	
	void setEditing(boolean flag)
	{
		firstClicked = null;
		editing = flag;
	
		for(int i = 0; i < children.size(); i++)
		{
			RemoteComponent current = (RemoteComponent)children.elementAt(i);
			current.setEditing(flag);
		}
	}
	
	void setGridVisible(boolean flag)
	{
		for(int i = 0; i < children.size(); i++)
		{
			RemoteComponent current = (RemoteComponent)children.elementAt(i);
			current.setGridVisible(flag);
		}
	}
	
	private void editClicked(SelectionEvent e)
	{
		RemoteComponent source = (RemoteComponent)e.getSource();
		source.setFocus();
		
		if(firstClicked == null)
		{
			firstClicked = source;
			source.setHighlighted(true);
		}
		else if(firstClicked == source)
		{
			firstClicked = null;
			source.setHighlighted(false);
		}
		else
		{
			int firstIndex = children.indexOf(firstClicked);
			int secondIndex = children.indexOf(source);
			children.setElementAt(firstClicked, secondIndex);
			children.setElementAt(source, firstIndex);
			
			source.swap(firstClicked);

			firstClicked.setHighlighted(false);
			firstClicked = null;
		}
	}
	
	RemoteButton makeButton(String id, int w, int h)
	{
		RemoteButton returnMe = new RemoteButton(makeParent(), id, gridWidth, gridHeight);
		initComponent(returnMe, w, h);
		
		return returnMe;
	}
	
	RemoteSlider makeSlider(String id, int w, int h)
	{
		RemoteSlider returnMe = new RemoteSlider(makeParent(), id, gridWidth, gridHeight);
		initComponent(returnMe, w, h);

		return returnMe;
	}

	RemoteLabel makeLabel(String id, int w, int h)
	{
		RemoteLabel returnMe = new RemoteLabel(makeParent(), id, gridWidth, gridHeight);
		initComponent(returnMe, w, h);
		
		return returnMe;
	}
	
	RemoteImage makeImage(String id, int w, int h)
	{
		RemoteImage returnMe = new RemoteImage(makeParent(), id, gridWidth, gridHeight);
		initComponent(returnMe, w, h);
		
		return returnMe;
	}

	RemoteBlank makeBlank(int w, int h)
	{
		RemoteBlank returnMe = new RemoteBlank(makeParent(), gridWidth, gridHeight);
		initComponent(returnMe, w, h);
		
		return returnMe;
	}

	Composite getContents() { return child; }
	 
	/**
	 * Get a {@link java.util.Vector} that contains all of this panel's
	 * RemoteComponents.
	 *
	 * @return the list of child RemoteComponents
	 */
	public Vector getComponents() { return children; }
	
	/**
	 * Get a {@link Hashtable} that contains the current layout
	 * of the panel's RemoteComponents.
	 *
	 * @return a Hashtable of {@link PersistentLayout} classes, one for
	 * each "child" of the panel
	 */
	public Hashtable getLayoutInfo()
	{
		Hashtable layout = new Hashtable();
		int x = 0, y = 0;
		
		for(int i = 0; i < children.size(); i++)
		{
			RemoteComponent current = (RemoteComponent)children.elementAt(i);
			
			if(!(current instanceof RemoteBlank))
			{
				PersistentLayout currentLayout = new PersistentLayout();
				currentLayout.viewable = current.isViewable();
				currentLayout.x = x;
				currentLayout.y = y;
				currentLayout.width = current.getWidth();
				currentLayout.height = current.getHeight();
				
				layout.put(current.getFunctionName(), currentLayout);
			}
			x++;
			if(x >= 5)
			{
				x = 0;
				y++;
			}
		}
		return layout;
	}
	
	/**
	 * Respond to a key being pressed.  This method is used by the SWT
	 * library and should not be called by users.
	 */
	public void keyPressed(KeyEvent e) { }

	/**
	 * Respond to a key being released.  This method is used by the SWT
	 * library and should not be called by users.
	 */
	public void keyReleased(KeyEvent e)
	{
		if(firstClicked == null) return;

		int width = firstClicked.getWidth(), height = firstClicked.getHeight();
		if(e.keyCode == SWT.ARROW_LEFT) width--;
		else if(e.keyCode == SWT.ARROW_RIGHT) width++;
		else if(e.keyCode == '\n' || e.keyCode == '\r')
		{
			firstClicked.setViewable(!firstClicked.isViewable());
			return;
		}
		
		if(width <= 0 || height <= 0) return;
		firstClicked.setWidth(width);
		firstClicked.setHeight(height);
		
		firstClicked.resize();
	}
}
